#!/usr/bin/ruby1.9
# -*- coding: utf-8 -*-

require 'socket'
include Socket::Constants

# From ruby.h
#   struct RBasic {
#     VALUE flags;
#     VALUE klass;
#   };
#
#   #define RSTRING_EMBED_LEN_MAX ((int)((sizeof(VALUE)*3)/sizeof(char)-1))
#   struct RString {
#     struct RBasic basic;
#     union {
#	      struct {
#	        long len;
#	        char *ptr;
#	        union {
#		        long capa;
#		        VALUE shared;
#	        } aux;
#	      } heap;
#	      char ary[RSTRING_EMBED_LEN_MAX + 1];
#     } as;
#   }
#};


# These two strings are identical as far as I can tell:
dissect "\xE5\xA5\xBD"
dissect "好"

# This has the same sequence of bytes and same encoding,
# but a different 'length' so there is MORE to a string
# than just a list of bytes and an encoding.  It
# prints the same in a terminal.  This sentence from
# Ruby-Doc is incomplete: "A String object holds and
# manipulates an arbitrary sequence of bytes, typically
# representing characters"
dissect ("" << 22909)

s = send_through_socket("\xE5\xA5\xBD")
dissect s
s.force_encoding("UTF-8")
dissect s

s = (0xC2.chr + 0xA2.chr)
s.force_encoding("UTF-8")
dissect s

#s = ""
#s << 229
#s << 165
#s << 189
#s.encode!("UTF-8")
#dissect s

BEGIN {
  def dissect(str)
    puts "#{str.class}: #{str.inspect}, len=#{str.length}, bytesize=#{str.bytesize}, encoding=#{str.encoding}"

    ordlist = ""
    begin
      (0..(str.length-1)).each do |i|
        ordlist += str[i].ord.to_s + " "
      end
    rescue
      ordlist = "N/A"
    end

    puts "  codepoints = #{str.codepoints.to_a}"
    puts "  bytes = #{str.bytes.to_a}"
    puts "  chars = #{str.chars.to_a}"
    puts "  ords  = #{ordlist}"
    puts
  end

  def send_through_socket(str)
    # Server setup
    server_socket = Socket.new(AF_INET, SOCK_STREAM, 0)
    server_socket.bind(Socket.pack_sockaddr_in(2200, 'localhost'))
    server_socket.listen(1)
    
    # Client connect
    client_socket = Socket.new(AF_INET, SOCK_STREAM, 0)
    #client_socket.external_encoding = "UTF-8"
    #puts client_socket.external_encoding.inspect
    #puts client_socket.internal_encoding.inspect
    client_socket.connect(Socket.pack_sockaddr_in(2200, '127.0.0.1'))
    
    # Server accept connection
    remote_socket, remote_addr = server_socket.accept

    # Client send data and close
    client_socket.puts str
    client_socket.close

    # Server receive data
    received_string = remote_socket.recvfrom(str.bytesize)[0]
    remote_socket.close
    server_socket.close

    return received_string
  end
}
